@@ -1,9 +1,9 @@
 # -*- coding: utf-8 -*-
-from ._vue import Vue, VueRouter, VueConfig
+from ._vue import Vue, VueRouter, VueConfig, VueMap
 
 __author__ = "eseunghwan"
 __email__ = "shlee0920@naver.com"
 __version__ = Vue.version
 __all__ = [
-    "Vue", "VueRouter", "VueConfig"
+    "Vue", "VueRouter", "VueConfig", "VueMap"
 ]
