@@ -6,10 +6,68 @@ from bottle import Bottle, static_file, json_dumps
 from ._assets import assets_dir
 
 
+def render_vue(vue_file) -> str:
+    rendered = [
+        str(vbuild.render(gv))
+        for gv in glob(vue_file)
+    ]
+
+    if len(rendered) > 1:
+        return "\n".join(rendered)
+    elif len(rendered) == 1:
+        return rendered[0]
+    else:
+        return ""
+
+def render_vue_as_html(vue_file:str, template_file:str, component_files:str, view_files:str = None, base_style:str = "", title:str = None):
+    if title in (None, ""):
+        title = os.path.splitext(os.path.basename(vue_file))[0]
+    elif "/" in title:
+        title = title.split("/")[-1]
+
+    vue_string = render_vue(vue_file)
+    components = render_vue(component_files)
+    views = "" if view_files == None else render_vue(view_files)
+
+    vue_script = """
+    <link rel="stylesheet" href="/pyvuejs/static/pyvuejs.css">
+    <script type="text/javascript" src="/pyvuejs/static/vue.min.js"></script>
+    <script type="text/javascript" src="/pyvuejs/static/axios.min.js"></script>
+    <script type="text/javascript" src="/pyvuejs/static/pyvuejs.router.js"></script>
+    <style>
+    """ + base_style + """
+    </style>
+    """ + components + """
+    """ + views + """
+    """ + vue_string + """
+    <""" + title + """ id="app" />
+    <script>
+        new Vue({ el: '""" + title + """' });
+    </script>"""
+
+    with open(template_file, "r", encoding = "utf-8") as tr:
+        template = tr.read()
+
+    return template.replace("{$project_name}", title).replace("<App/>", vue_script).replace("<App></App>", vue_script)
+
+def route_vue(bottle:Bottle, endpoint:str, vue_file:str, template_file:str, component_files:str, view_files:str = None, base_style:str = ""):
+    endpoint = endpoint if endpoint.startswith("/") else "/" + endpoint
+    bottle.route(endpoint, callback = lambda: render_vue_as_html(vue_file, template_file, component_files, view_files, base_style, endpoint[1:]))
+
+
 class VueRouter(list):
     def __init__(self, routes:list):
         super().__init__(routes)
 
+    def register(self, bottle:Bottle, app_dir:str):
+        for route in self:
+            route_vue(
+                bottle, "/routes" + route["path"],
+                os.path.join(app_dir, "src", "views", route["component"] + ".vue"),
+                os.path.join(app_dir, "public", "index.html"), os.path.join(app_dir, "src", "components", "*.vue"),
+                base_style = vbuild.render(os.path.join(app_dir, "App.vue")).style
+            )
+
 class VueConfig:
     # pyvuejs -> pvuejs -> 047372 -> 47372
     def __init__(self, host:str = "0.0.0.0", port:int = 47372, debug:bool = True, open_webbrowser:bool = True):
@@ -36,61 +94,48 @@ class VueMap:
             except:
                 return json_dumps("")
 
-    def register(self, bottle:Bottle):
-        bottle.route(f"/{self.__class__.__name__}/<callback_name>", method = ["GET", "POST"], callback = lambda callback_name: self.__callback_to_response(callback_name))
-            
+    def register(self, bottle:Bottle, debug:bool = False):
+        bottle.route(
+            f"/{self.__class__.__name__}/<callback_name>",
+            method = ["GET", "POST"] if debug else "POST",
+            callback = lambda callback_name: self.__callback_to_response(callback_name)
+        )
 
 
 class Vue:
-    version:str = "2.0.3"
+    version:str = "2.0.4"
+    router:VueRouter = VueRouter([])
+    config:VueConfig = VueConfig()
 
     def __init__(self):
         vbuild.fullPyComp = True
-        self.__router, self.__config = VueRouter([]), VueConfig()
         self.__bottle = Bottle()
         self.__bottle.route("/stop", callback = lambda: os.kill(os.getpid(), signal.SIGTERM))
 
-    def use(self, obj):
+    @staticmethod
+    def use(obj):
         if isinstance(obj, VueRouter):
-            self.__router = obj
+            Vue.router = obj
         elif isinstance(obj, VueConfig):
-            self.__config = obj
-
-        return self
+            Vue.config = obj
 
     def map(self, callback, method:str = "GET", group:str = "fn"):
         self.__bottle.route(f"/{group}/{callback.__name__}", method = method.upper(), callback = lambda: json_dumps(callback()))
 
+
     def __load_project(self, app_dir:str) -> str:
         public_dir = os.path.join(app_dir, "public")
         self.__load_publics(public_dir)
 
         self.__load_assets(os.path.join(app_dir, "src", "assets"))
         self.__load_maps(os.path.join(app_dir, "src", "maps"))
+        self.__load_router(os.path.join(app_dir, "src"))
 
-        @self.__bottle.route("/")
-        def route_app():
-            with open(os.path.join(public_dir, "index.html"), encoding = "utf-8") as tr:
-                template = tr.read()
-
-            components = self.__load_components(os.path.join(app_dir, "src", "components"))
-            views = self.__load_views(os.path.join(app_dir, "src", "views"))
-            main_app = str(vbuild.render(os.path.join(app_dir, "App.vue")))
-            app_script = """
-            <script type="text/javascript" src="/pyvuejs/static/vue.min.js"></script>
-            <script type="text/javascript" src="/pyvuejs/static/axios.min.js"></script>
-            """ + components + """
-            """ + views + """
-            """ + main_app + """
-            <App/>
-            <script>
-            new Vue({ el: "app" })
-            </script>"""
-
-            return template.replace(
-                "{$project_name}", os.path.basename(app_dir)
-            ).replace("<App/>", app_script).replace("<App></App>", app_script)
-
+        route_vue(
+            self.__bottle, "/", os.path.join(app_dir, "App.vue"),
+            os.path.join(public_dir, "index.html"),
+            os.path.join(app_dir, "src", "components", "*.vue"), os.path.join(app_dir, "src", "views", "*.vue")
+        )
 
     def __load_publics(self, project_public_dir:str):
         if os.path.exists(os.path.join(project_public_dir, "favicon.ico")):
@@ -104,12 +149,6 @@ class Vue:
     def __load_assets(self, project_assets_dir:str):
         self.__bottle.route("/assets/<asset_file_path:path>", callback = lambda asset_file_path: static_file(asset_file_path, project_assets_dir))
 
-    def __load_components(self, project_components_dir:str) -> str:
-        return str(vbuild.render(os.path.join(project_components_dir, "*.vue")))
-
-    def __load_views(self, project_views_dir:str) -> str:
-        return str(vbuild.render(os.path.join(project_views_dir, "*.vue")))
-
     def __load_maps(self, project_maps_dir:str):
         sys.path.append(project_maps_dir)
         for map_file in glob(os.path.join(project_maps_dir, "*.py")):
@@ -120,6 +159,13 @@ class Vue:
 
         sys.path.remove(project_maps_dir)
 
+    def __load_router(self, project_src_dir:str):
+        sys.path.append(project_src_dir)
+        importlib.import_module("router")
+        sys.path.remove(project_src_dir)
+
+        self.router.register(self.__bottle, os.path.dirname(project_src_dir))
+
     def serve(self):
         @self.__bottle.route("/pyvuejs/static/<asset_file_path:path>")
         def get_asset_file(asset_file_path:str):
@@ -133,15 +179,15 @@ class Vue:
 
         self.__load_project(os.getcwd())
 
-        if self.__config.open_webbrowser:
-            webbrowser.open_new(f"http://127.0.0.1:{self.__config.port}/")
+        if self.config.open_webbrowser:
+            webbrowser.open_new(f"http://127.0.0.1:{self.config.port}/")
 
         self.__serve()
 
     def __serve(self):
         try:
-            self.__bottle.run(host = self.__config.host, port = self.__config.port, quiet = not self.__config.debug)
+            self.__bottle.run(host = self.config.host, port = self.config.port, quiet = not self.config.debug)
         except OSError:
             if "address already in use" in str(sys.exc_info()[1]).lower():
-                http.client.HTTPConnection("127.0.0.1", self.__config.port).request("GET", "/stop")
+                http.client.HTTPConnection("127.0.0.1", self.config.port).request("GET", "/stop")
                 self.__serve()
